require 'optparse'

module Notebook
  class Options
    DEFAULT_NOTEBOOK = "notebook.txt"
    attr_reader :notebook, :add, :read, :edit, :edit_number, :all, :all_title

    def initialize(argv)
      @notebook = DEFAULT_NOTEBOOK
      parse(argv)
    end

    private
    def parse(argv)
      @add = false
      @read = false
      @edit = false
      @all = false
      OptionParser.new do |opts|
        opts.banner = "Usage: notebook [ options ]"

        opts.on("-n", "--notebook PATH", String, "Path to notebook file") do |notebook|
          @notebook = notebook
        end

        opts.on("-a", "--add", "Add a note") { @add = true }

        opts.on("-r", "--read", "Read back notes") { @read = true }

        opts.on("-e", "--edit NUMBER", Integer, "Edit a specific note") do |number|
          @edit = true
          @edit_number = number
        end

        opts.on("-x", "--do_to_all TITLE", String, 
                "replace all note titles with TITLE") do |title|  
          @all = true 
          @all_title = title
        end
       
        opts.on_tail("-h", "--help", "Show this message") do
          puts opts
          exit
        end

        begin      
          opts.parse!(argv)
        rescue OptionParser::InvalidOption => e
          puts e
          puts opts
          exit(1)
        end
      end
    end
  end
end

